@@ -29,7 +29,7 @@
 #define SIZEOF_VOIDP 8
 #define SIZEOF_FLOAT 4
 #define SIZEOF_DOUBLE 8
-#define SIZEOF_TIME_T 4
+#define SIZEOF_TIME_T 8
 #define PRI_LL_PREFIX "ll"
 #define rb_pid_t pid_t
 #define PIDT2NUM(v) INT2NUM(v)
@@ -44,9 +44,9 @@
 #define NUM2GIDT(v) NUM2UINT(v)
 #define PRI_GIDT_PREFIX PRI_INT_PREFIX
 #define rb_time_t time_t
-#define TIMET2NUM(v) INT2NUM(v)
-#define NUM2TIMET(v) NUM2INT(v)
-#define PRI_TIMET_PREFIX PRI_INT_PREFIX
+#define TIMET2NUM(v) LL2NUM(v)
+#define NUM2TIMET(v) NUM2LL(v)
+#define PRI_TIMET_PREFIX PRI_LL_PREFIX
 #define rb_dev_t dev_t
 #define DEVT2NUM(v) INT2NUM(v)
 #define NUM2DEVT(v) NUM2INT(v)
@@ -256,13 +256,17 @@
 #define HAVE_STRUCT_TM_TM_ZONE 1
 #define HAVE_TM_ZONE 1
 #define HAVE_STRUCT_TM_TM_GMTOFF 1
+#define HAVE_DAYLIGHT 1
+#define HAVE_VAR_TIMEZONE 1
+#define TYPEOF_VAR_TIMEZONE long
 #define HAVE_TIMEZONE 1
+#define TIMEZONE_VOID 1
 #define NEGATIVE_TIME_T 1
 #define POSIX_SIGNAL 1
 #define RSHIFT(x,y) ((x)>>(int)(y))
 #define FILE_COUNT _r
 #define FILE_READPTR _p
-#define SIZEOF_STRUCT_STAT_ST_INO 4
+#define SIZEOF_STRUCT_STAT_ST_INO SIZEOF_LONG
 #define HAVE__SC_CLK_TCK 1
 #define STACK_GROW_DIRECTION -1
 #define _REENTRANT 1
