@@ -13,17 +13,19 @@ class Gem::Ext::Builder
 
   def self.make(dest_path, results)
     unless File.exist? 'Makefile' then
-      raise Gem::InstallError, "Makefile not found:\n\n#{results.join "\n"}" 
+      raise Gem::InstallError, "Makefile not found:\n\n#{results.join "\n"}"
     end
 
     mf = File.read('Makefile')
     mf = mf.gsub(/^RUBYARCHDIR\s*=\s*\$[^$]*/, "RUBYARCHDIR = #{dest_path}")
     mf = mf.gsub(/^RUBYLIBDIR\s*=\s*\$[^$]*/, "RUBYLIBDIR = #{dest_path}")
-    mf = mf.gsub(/-o root -g bin/, "") if Gem::Installer.path_warning
+    mf = mf.gsub(/-o root -g bin/, "") unless Process.euid == 0
 
     File.open('Makefile', 'wb') {|f| f.print mf}
 
-    make_program = ENV['make']
+    # try to find make program from Ruby configure arguments first
+    RbConfig::CONFIG['configure_args'] =~ /with-make-prog\=(\w+)/
+    make_program = ENV['MAKE'] || ENV['make'] || $1
     unless make_program then
       make_program = (/mswin/ =~ RUBY_PLATFORM) ? 'nmake' : 'make'
     end
