@@ -1,7 +1,6 @@
 #
-#		thread.rb - thread support classes
-#			$Date: 2005/06/07 09:41:17 $
-#			by Yukihiro Matsumoto <matz@netlab.co.jp>
+#               thread.rb - thread support classes
+#                       by Yukihiro Matsumoto <matz@netlab.co.jp>
 #
 # Copyright (C) 2001  Yukihiro Matsumoto
 # Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
@@ -9,11 +8,11 @@
 #
 
 unless defined? Thread
-  fail "Thread not available for this ruby interpreter"
+  raise "Thread not available for this ruby interpreter"
 end
 
 unless defined? ThreadError
-  class ThreadError<StandardError
+  class ThreadError < StandardError
   end
 end
 
@@ -21,144 +20,7 @@ if $DEBUG
   Thread.abort_on_exception = true
 end
 
-class Thread
-  #
-  # Wraps a block in Thread.critical, restoring the original value upon exit
-  # from the critical section.
-  #
-  def Thread.exclusive
-    _old = Thread.critical
-    begin
-      Thread.critical = true
-      return yield
-    ensure
-      Thread.critical = _old
-    end
-  end
-end
-
 #
-# Mutex implements a simple semaphore that can be used to coordinate access to
-# shared data from multiple concurrent threads.
-#
-# Example:
-#
-#   require 'thread'
-#   semaphore = Mutex.new
-#   
-#   a = Thread.new {
-#     semaphore.synchronize {
-#       # access shared resource
-#     }
-#   }
-#   
-#   b = Thread.new {
-#     semaphore.synchronize {
-#       # access shared resource
-#     }
-#   }
-#
-class Mutex
-  #
-  # Creates a new Mutex
-  #
-  def initialize
-    @waiting = []
-    @locked = false;
-    @waiting.taint		# enable tainted comunication
-    self.taint
-  end
-
-  #
-  # Returns +true+ if this lock is currently held by some thread.
-  #
-  def locked?
-    @locked
-  end
-
-  #
-  # Attempts to obtain the lock and returns immediately. Returns +true+ if the
-  # lock was granted.
-  #
-  def try_lock
-    result = false
-    Thread.critical = true
-    unless @locked
-      @locked = true
-      result = true
-    end
-    Thread.critical = false
-    result
-  end
-
-  #
-  # Attempts to grab the lock and waits if it isn't available.
-  #
-  def lock
-    while (Thread.critical = true; @locked)
-      @waiting.push Thread.current
-      Thread.stop
-    end
-    @locked = true
-    Thread.critical = false
-    self
-  end
-
-  #
-  # Releases the lock. Returns +nil+ if ref wasn't locked.
-  #
-  def unlock
-    return unless @locked
-    Thread.critical = true
-    @locked = false
-    begin
-      t = @waiting.shift
-      t.wakeup if t
-    rescue ThreadError
-      retry
-    end
-    Thread.critical = false
-    begin
-      t.run if t
-    rescue ThreadError
-    end
-    self
-  end
-
-  #
-  # Obtains a lock, runs the block, and releases the lock when the block
-  # completes.  See the example under Mutex.
-  #
-  def synchronize
-    lock
-    begin
-      yield
-    ensure
-      unlock
-    end
-  end
-
-  #
-  # If the mutex is locked, unlocks the mutex, wakes one waiting thread, and
-  # yields in a critical section.
-  #
-  def exclusive_unlock
-    return unless @locked
-    Thread.exclusive do
-      @locked = false
-      begin
-	t = @waiting.shift
-	t.wakeup if t
-      rescue ThreadError
-	retry
-      end
-      yield
-    end
-    self
-  end
-end
-
-# 
 # ConditionVariable objects augment class Mutex. Using condition variables,
 # it is possible to suspend while in the middle of a critical section until a
 # resource becomes available.
@@ -169,7 +31,7 @@ end
 #
 #   mutex = Mutex.new
 #   resource = ConditionVariable.new
-#   
+#
 #   a = Thread.new {
 #     mutex.synchronize {
 #       # Thread 'a' now needs the resource
@@ -177,7 +39,7 @@ end
 #       # 'a' can now have the resource
 #     }
 #   }
-#   
+#
 #   b = Thread.new {
 #     mutex.synchronize {
 #       # Thread 'b' has finished using the resource
@@ -191,49 +53,60 @@ class ConditionVariable
   #
   def initialize
     @waiters = []
+    @waiters_mutex = Mutex.new
   end
-  
+
   #
   # Releases the lock held in +mutex+ and waits; reacquires the lock on wakeup.
   #
-  def wait(mutex)
+  # If +timeout+ is given, this method returns after +timeout+ seconds passed,
+  # even if no other thread doesn't signal.
+  #
+  def wait(mutex, timeout=nil)
     begin
-      mutex.exclusive_unlock do
+      # TODO: mutex should not be used
+      @waiters_mutex.synchronize do
         @waiters.push(Thread.current)
-        Thread.stop
       end
+      mutex.sleep timeout
     ensure
-      mutex.lock
+      @waiters_mutex.synchronize do
+        @waiters.delete(Thread.current)
+      end
     end
+    self
   end
-  
+
   #
   # Wakes up the first thread in line waiting for this lock.
   #
   def signal
     begin
-      t = @waiters.shift
+      t = @waiters_mutex.synchronize { @waiters.shift }
       t.run if t
     rescue ThreadError
       retry
     end
+    self
   end
-    
+
   #
   # Wakes up all threads waiting for this lock.
   #
   def broadcast
+    # TODO: incomplete
     waiters0 = nil
-    Thread.exclusive do
+    @waiters_mutex.synchronize do
       waiters0 = @waiters.dup
       @waiters.clear
     end
     for t in waiters0
       begin
-	t.run
+        t.run
       rescue ThreadError
       end
     end
+    self
   end
 end
 
@@ -243,9 +116,9 @@ end
 # Example:
 #
 #   require 'thread'
-#   
+#
 #   queue = Queue.new
-#   
+#
 #   producer = Thread.new do
 #     5.times do |i|
 #       sleep rand(i) # simulate expense
@@ -253,7 +126,7 @@ end
 #       puts "#{i} produced"
 #     end
 #   end
-#   
+#
 #   consumer = Thread.new do
 #     5.times do |i|
 #       value = queue.pop
@@ -261,7 +134,7 @@ end
 #       puts "consumed #{value}"
 #     end
 #   end
-#   
+#
 #   consumer.join
 #
 class Queue
@@ -271,29 +144,25 @@ class Queue
   def initialize
     @que = []
     @waiting = []
-    @que.taint		# enable tainted comunication
+    @que.taint          # enable tainted communication
     @waiting.taint
     self.taint
+    @mutex = Mutex.new
   end
 
   #
   # Pushes +obj+ to the queue.
   #
   def push(obj)
-    Thread.critical = true
-    @que.push obj
-    begin
-      t = @waiting.shift
-      t.wakeup if t
-    rescue ThreadError
-      retry
-    ensure
-      Thread.critical = false
-    end
-    begin
-      t.run if t
-    rescue ThreadError
-    end
+    @mutex.synchronize{
+      @que.push obj
+      begin
+        t = @waiting.shift
+        t.wakeup if t
+      rescue ThreadError
+        retry
+      end
+    }
   end
 
   #
@@ -312,14 +181,17 @@ class Queue
   # thread isn't suspended, and an exception is raised.
   #
   def pop(non_block=false)
-    while (Thread.critical = true; @que.empty?)
-      raise ThreadError, "queue empty" if non_block
-      @waiting.push Thread.current
-      Thread.stop
-    end
-    @que.shift
-  ensure
-    Thread.critical = false
+    @mutex.synchronize{
+      while true
+        if @que.empty?
+          raise ThreadError, "queue empty" if non_block
+          @waiting.push Thread.current
+          @mutex.sleep
+        else
+          return @que.shift
+        end
+      end
+    }
   end
 
   #
@@ -333,7 +205,7 @@ class Queue
   alias deq pop
 
   #
-  # Returns +true+ is the queue is empty.
+  # Returns +true+ if the queue is empty.
   #
   def empty?
     @que.empty?
@@ -372,7 +244,7 @@ end
 #
 # See Queue for an example of how a SizedQueue works.
 #
-class SizedQueue<Queue
+class SizedQueue < Queue
   #
   # Creates a fixed-length queue with a maximum size of +max+.
   #
@@ -380,7 +252,7 @@ class SizedQueue<Queue
     raise ArgumentError, "queue size must be positive" unless max > 0
     @max = max
     @queue_wait = []
-    @queue_wait.taint		# enable tainted comunication
+    @queue_wait.taint           # enable tainted comunication
     super()
   end
 
@@ -395,21 +267,23 @@ class SizedQueue<Queue
   # Sets the maximum size of the queue.
   #
   def max=(max)
-    Thread.critical = true
-    if max <= @max
-      @max = max
-      Thread.critical = false
-    else
-      diff = max - @max
-      @max = max
-      Thread.critical = false
+    diff = nil
+    @mutex.synchronize {
+      if max <= @max
+        @max = max
+      else
+        diff = max - @max
+        @max = max
+      end
+    }
+    if diff
       diff.times do
-	begin
-	  t = @queue_wait.shift
-	  t.run if t
-	rescue ThreadError
-	  retry
-	end
+        begin
+          t = @queue_wait.shift
+          t.run if t
+        rescue ThreadError
+          retry
+        end
       end
     end
     max
@@ -420,13 +294,21 @@ class SizedQueue<Queue
   # until space becomes available.
   #
   def push(obj)
-    Thread.critical = true
-    while @que.length >= @max
-      @queue_wait.push Thread.current
-      Thread.stop
-      Thread.critical = true
-    end
-    super
+    @mutex.synchronize{
+      while true
+        break if @que.length < @max
+        @queue_wait.push Thread.current
+        @mutex.sleep
+      end
+
+      @que.push obj
+      begin
+        t = @waiting.shift
+        t.wakeup if t
+      rescue ThreadError
+        retry
+      end
+    }
   end
 
   #
@@ -444,21 +326,16 @@ class SizedQueue<Queue
   #
   def pop(*args)
     retval = super
-    Thread.critical = true
-    if @que.length < @max
-      begin
-	t = @queue_wait.shift
-	t.wakeup if t
-      rescue ThreadError
-	retry
-      ensure
-	Thread.critical = false
+    @mutex.synchronize {
+      if @que.length < @max
+        begin
+          t = @queue_wait.shift
+          t.wakeup if t
+        rescue ThreadError
+          retry
+        end
       end
-      begin
-	t.run if t
-      rescue ThreadError
-      end
-    end
+    }
     retval
   end
 
@@ -478,6 +355,22 @@ class SizedQueue<Queue
   def num_waiting
     @waiting.size + @queue_wait.size
   end
+
+  #
+  # Removes all objects from the queue and wakes waiting threads, if any.
+  #
+  def clear
+    @mutex.synchronize do
+      @que.clear
+      begin
+        until @queue_wait.empty?
+          @queue_wait.shift.wakeup
+        end
+      rescue ThreadError
+        retry
+      end
+    end
+  end
 end
 
 # Documentation comments:
