@@ -13,8 +13,8 @@ module RbConfig
   CONFIG["MAJOR"] = "1"
   CONFIG["MINOR"] = "9"
   CONFIG["TEENY"] = "1"
-  CONFIG["PATCHLEVEL"] = "545"
-  CONFIG["INSTALL"] = '/usr/bin/install -c -o root -g bin'
+  CONFIG["PATCHLEVEL"] = "551"
+  CONFIG["INSTALL"] = '/usr/obj/ruby-1.9.3-p551/bin/install -c '
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "ruby19"
@@ -25,14 +25,14 @@ module RbConfig
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " newline.c"
   CONFIG["USE_RUBYGEMS"] = "YES"
-  CONFIG["MANTYPE"] = "man"
-  CONFIG["NROFF"] = "/bin/false"
+  CONFIG["MANTYPE"] = "doc"
+  CONFIG["NROFF"] = ""
   CONFIG["vendorhdrdir"] = "$(rubyhdrdir)/vendor_ruby"
   CONFIG["sitehdrdir"] = "$(rubyhdrdir)/site_ruby"
   CONFIG["rubyhdrdir"] = "$(includedir)/$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--program-suffix=19' '--with-soname=ruby19' '--enable-pthread' '--enable-ipv6' '--disable-option-checking' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--enable-shared' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--program-suffix=19' '--with-soname=ruby19' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--disable-option-checking' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--enable-shared' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["vendordir"] = "$(rubylibprefix)/vendor_ruby"
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["ruby_version"] = "1.9.1"
@@ -113,9 +113,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "install -c -o root -g bin -m 444"
-  CONFIG["INSTALL_SCRIPT"] = "install -c -o root -g bin -m 555"
-  CONFIG["INSTALL_PROGRAM"] = "install -c -s -o root -g bin -m 555"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ruby-1.9.3-p551/bin/install -c  -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ruby-1.9.3-p551/bin/install -c  -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ruby-1.9.3-p551/bin/install -c -s  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -143,21 +143,21 @@ module RbConfig
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -Wl,-E"
   CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd5.6"
+  CONFIG["target_os"] = "openbsd5.7"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "x86_64"
-  CONFIG["target"] = "x86_64-unknown-openbsd5.6"
-  CONFIG["host_os"] = "openbsd5.6"
+  CONFIG["target"] = "x86_64-unknown-openbsd5.7"
+  CONFIG["host_os"] = "openbsd5.7"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "x86_64"
-  CONFIG["host"] = "x86_64-unknown-openbsd5.6"
+  CONFIG["host"] = "x86_64-unknown-openbsd5.7"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd5.6"
+  CONFIG["build_os"] = "openbsd5.7"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "x86_64"
-  CONFIG["build"] = "x86_64-unknown-openbsd5.6"
-  CONFIG["RUBY_RELEASE_DATE"] = "2014-02-24"
+  CONFIG["build"] = "x86_64-unknown-openbsd5.7"
+  CONFIG["RUBY_RELEASE_DATE"] = "2014-11-13"
   CONFIG["RUBY_PROGRAM_VERSION"] = "1.9.3"
   CONFIG["BASERUBY"] = "echo executable host ruby is required.  use --with-baseruby option.; false"
   CONFIG["target_alias"] = ""
@@ -238,3 +238,23 @@ module RbConfig
 end
 autoload :Config, "rbconfig/obsolete.rb" # compatibility for ruby-1.8.4 and older.
 CROSS_COMPILING = nil unless defined? CROSS_COMPILING
+install = '/usr/bin/install -c -o root -g bin'  
+c = RbConfig::CONFIG 
+m = RbConfig::MAKEFILE_CONFIG
+if v = ENV['BSD_INSTALL_SCRIPT']
+  m['INSTALL'] = c['INSTALL'] = v.sub('-m 755', '')
+  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = v
+else
+  m['INSTALL'] = c['INSTALL'] = "#{install}"
+  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = "#{install} -m 755"
+end
+if v = ENV['BSD_INSTALL_PROGRAM']
+  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = v
+else
+  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = "#{install} -s -m 755"
+end
+if v = ENV['BSD_INSTALL_DATA']
+  m['INSTALL_DATA'] = c['INSTALL_DATA'] = v
+else
+  m['INSTALL_DATA'] = c['INSTALL_DATA'] = "#{install} -m 644"
+end
